using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Management.Automation.Runspaces;
using System.Management.Automation;
using System.Text;
using System.Windows.Forms;
using System.Security;


namespace ShinyPower
{
    public partial class frmHelp : Form
    {

        //Runspace runSpace = RunspaceFactory.CreateRunspace();
            
            

        private BackgroundWorker m_Worker = null;
        #region enums
        public enum shinyOutputTarget
        {
            help,
            syntax,
            listbox,
            combo,
            none
        }        
    
        private enum shinyHelpMode
        {
            cmdLets,
            Alias,
            About
        }
        #endregion

        #region private members


        private Stack<Cursor> cursorStack = new Stack<Cursor>();

        private shinyHelpMode m_mode = shinyHelpMode.cmdLets;

        #endregion
        #region private properties

        private bool syntaxVisible
        {
            set
            {
                splitContainer1.Panel1Collapsed = !value;
            }
        }

        private shinyHelpMode Mode
        {
            get { return m_mode; }
            set { m_mode = value;

            PopulateListBox(m_mode);
            }
        }
        #endregion

        public frmHelp()
        {
            InitializeComponent();
        }
        
        private void frmHelp_Load(object sender, EventArgs e)
        {
            
              
            //cboCmdLet.SelectedValue = "cmdLet topics";
            cboCmdLet.SelectedIndex = 0; 
        }

        private void PopulateListBox(shinyHelpMode mode)
        {
            string cmdText = string.Empty;
            

            if (mode == shinyHelpMode.cmdLets)
            {
                cmdText = "get-command";
                
            }
            if (mode == shinyHelpMode.About)
            {
                cmdText = "get-help about* | format-list name | out-string";
            }

            if (mode == shinyHelpMode.Alias)
            {
                cmdText = "alias  | format-list name | out-string";
                   //"get-help * | where ({-not ($_.name -like 'about*')}) | format-list name | out-string";       
            }

             
            Runspace runSpace = RunspaceFactory.CreateRunspace();
            runSpace.Open();
            lstCmdLets.Items.Clear();
            Pipeline cmd = runSpace.CreatePipeline(cmdText);

            Collection<PSObject> resultObject = cmd.Invoke();

            if (mode != shinyHelpMode.cmdLets)
            {
                string[] names = System.Text.RegularExpressions.Regex.Split(resultObject[0].ToString(), "\r\nName : ");
                foreach (string name in names)
                {
                    if (name.Trim() != string.Empty)
                    {
                        lstCmdLets.Items.Add(name.Trim());
                    }
                }

            }
            else
            {
                foreach (PSObject ps1 in resultObject)
                {
                    lstCmdLets.Items.Add(ps1.ToString());
                }
            }
            if (cmd.PipelineStateInfo.State == PipelineState.Completed)
            {
                cmd.Stop(); 
            }

        }


        #region menu actions
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {

            using (frmAbout frm = new frmAbout())
            {
                frm.ShowDialog();
            }

        }
        #endregion

        #region populate listbox


        /// <summary>
        /// Handles the SelectedIndexChanged event of the cboCmdLet control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
        private void cboCmdLet_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cboCmdLet.SelectedItem.ToString())
            {
                case @"""about"" topics":
                    Mode = shinyHelpMode.About;
                    break;
                case "cmdLet topics":
                    Mode = shinyHelpMode.cmdLets;
                    break;
                case "aliases":
                    Mode = shinyHelpMode.Alias;
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the lstCmdLets control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
        private void lstCmdLets_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstCmdLets.Text == string.Empty) return;
            switch (m_mode)
            {
                case shinyHelpMode.About:
                    showAbout(lstCmdLets.Text);
                    break;
                case shinyHelpMode.cmdLets:
                    showCmdLets(lstCmdLets.Text);
                    break;
                case shinyHelpMode.Alias:
                    showAlias(lstCmdLets.Text);
                    break;
                default:
                    MessageBox.Show("Don't know how to display that", this.Text); 
                    break;
            }
        }

        /// <summary>
        /// Shows the CMD lets.
        /// </summary>
        /// <param name="cmdLetName">Name of the CMD let.</param>
        private void showCmdLets(string cmdLetName) {
            syntaxVisible = true;
            InvokePSCommand("get-Command " + cmdLetName + " | Format-List | out-string", shinyOutputTarget.syntax );
            InvokePSCommand("get-Help " + cmdLetName + " | out-string", shinyOutputTarget.help);
        }

        /// <summary>
        /// Shows the about.
        /// </summary>
        /// <param name="cmdLetName">Name of the CMD let.</param>
        private void showAbout(string cmdLetName)
        {
            syntaxVisible = false;
            InvokePSCommand("get-Help " + cmdLetName + " | out-string", shinyOutputTarget.help);
        }

        private void showAlias(string cmdLetName)
        {
            syntaxVisible = false;
            InvokePSCommand("get-Help " + cmdLetName + " | out-string", shinyOutputTarget.help);
        }

        #endregion

        #region invoke command (on threads)
        public void InvokePSCommand(string commandText, shinyOutputTarget target)
        {
            cursorStack.Push(this.Cursor);
            this.Cursor = Cursors.WaitCursor;

            Runspace runSpace = RunspaceFactory.CreateRunspace();

            runSpace.Open();
            

            Pipeline cmd = runSpace.CreatePipeline(commandText);
            m_Worker = new BackgroundWorker();
            m_Worker.DoWork += new DoWorkEventHandler(InvokeWorker_DoWork);

            
            m_Worker.WorkerReportsProgress = false;            
            TextBox tb = null;
            switch (target)
            {
                case shinyOutputTarget.help:                    
                    lblHelp.Text = string.Format("HELP     ('{0}')", commandText);
                    tb = txtHelp;                    
                    break;
                case shinyOutputTarget.syntax:                    
                    lblSyntax.Text = string.Format("SYNTAX      ('{0}')", commandText);
                    tb = txtSyntax;                    
                    break;
                default:
                    break;
            
            }
            if (tb != null)
            {
                tb.Text = "";
                m_Worker.RunWorkerCompleted +=
                    delegate(object sender, RunWorkerCompletedEventArgs e)
                    {
                        if (e.Error == null)
                        {
                            Collection<PSObject> resultObject = e.Result as Collection<PSObject>;
                            tb.Text = resultObject[0].ToString().Trim();
                        }
                        this.Cursor = cursorStack.Pop();
                    };

            }
            object[] workerParams = new object[1];
            workerParams[0] = cmd;            
            m_Worker.RunWorkerAsync(workerParams);
        }

        private void InvokeWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            object[] workerParams = (object[])e.Argument;

            Collection<PSObject> resultObject =   ((Pipeline)workerParams[0]).Invoke();
            e.Result = resultObject;

            ((Pipeline)workerParams[0]).Runspace.Close();
            
        }


       

        private void InvokeWorker_RunWorkerCompletedHelp(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                Collection<PSObject> resultObject = e.Result as Collection<PSObject>;                
                txtHelp.Text = resultObject[0].ToString().Trim();
            }
            this.Cursor = cursorStack.Pop();         
            
        }

        private void InvokeWorker_RunWorkerCompletedSyntax(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                Collection<PSObject> resultObject = e.Result as Collection<PSObject>;
                txtSyntax.Text = resultObject[0].ToString().Trim();
            }            
            this.Cursor = cursorStack.Pop();
        }
        #endregion

    }
}